/* I-7188XB-512-CAN:Use 7188XB to send PDO to control DO of 2054C,and read DI value																			

																			(1)Write DO value 

																			(2)Set event time 

																			(3)Reset event time 

																			I-7188XB-512 + CAN-2054C

		node ID :0x01 
		
		Baud Rate Rotary Switch: 3 (125 kbps) 

    Compiler: TC++ 3.0

    Compile mode: large

    Project: PDO.c

             7188xbl.Lib

             xc100l.Lib 
             
    Communication: CANBus of 7188XB <==>CANopen<==>CANBus of 2054C 
    
    Hardware: I-7188XB-512 + CAN-2054C   

    RxPDO Communication Entry:

		 Idx  Sidx  Description           Type        Attr  Default 

    1400h  0h   Number of entries     UNSIGNED 8  RO    2 

           1h   COB-ID used by RxPDO  UNSIGNED 32 RW    200h+x 

           2h   Transmission type     UNSIGNED 8  RW    FFh            

    RxPDO Mapping Communication Entry:

			Idx  Sidx  Description             Type         Attr  Default 

		1600h  0h  	 Number of entries       UNSIGNED 8   RW    8 

  				 1h    Write Output channel 0  UNSIGNED 32  RW    6200 0108h  				 

  	TxPDO Communication Entry: 

		Idx   Sidx  Description           Type         Attr  Default 

		1800h  0h   Number of entries     UNSIGNED 8   RO    5 

  				 1h   COB-ID used by TxPDO  UNSIGNED 32  RW    180h+x 

  				 2h   Transmission type     UNSIGNED 8   RW    FFh 

           3h   Inhibit time          UNSIGNED 16        0 

           4h   reversed              ---          ---   --- 

           5h   Event timer           UNSIGNED 16        0   			           

    TxPDO Mapping Communication Entry:

		Idx  	 Sidx  Description        Type         Attr  Default 

		1A00h  0h  	 Number of entries  UNSIGNED 8   RO    1 

  	       1h  	 DI channel 0 ~ 7   UNSIGNED 32  RW    6000 0108h

[10,Dec,2011] by Alan

*/

#include <stdlib.h>

#include "..\..\lib\7188xb.h"

#include "..\..\lib\xc100.h"

unsigned char MsgData[8];
unsigned int PrintFlag = 1;
unsigned long MsgCnt=0;

/*Receive CAN message */

void ReceiveCANData(void);

/*Send SDO (COB-ID form client to server (RxSDO),COB-ID:600h + node ID)*/

void SendCANSDO(unsigned char cmd,unsigned char index_l,unsigned char index_h,unsigned char sub_index,

						    unsigned char D1,unsigned char D2,unsigned char D3,unsigned char D4);

/*send PDO (COB-ID (200h+node ID) used by RxPDO)*/

void SendCANPDO(unsigned char D1);

/*Send NMT (COB-ID : 0),Send NMT Error Control(COB-ID:700h + node ID)*/

void SendCANNMT(unsigned char RTR,unsigned char Cmd);

/*user define interrupt */



void UserCANInt(char CANInt){  

  if (CANInt) ;   /*avoid complier warning*/

  if (CANInt==0x01){ /*for Rx interrupt*/

  	L3On();
		if(PrintFlag)
  	ReceiveCANData();			

  }

  else if (CANInt==0x02){ /*for Tx interrupt*/

  	L2On();  

  }   

  L2Off();

  L3Off();

}


void main(void)
{

		int ret;		

		unsigned short _hexValue2=0,_hexValue=0,_choice=0,_quit;

		_quit = 1;

		//char MsgDataTemp[1];	

    InitLib();


    /********************************************************/

    /*                    Initial all LEDs                  */

    /********************************************************/

    L1Off();

    L2Off();

    L3Off();   

    /*******************************************************/

    /*      initialize and configure the CAN controller    */

    /*******************************************************/		

    

   ret=XC100Init(0,3,125000UL,0,0,0x00000000UL,0xffffffffUL);

    /* XCANInit function Parameter descriptions

       1            : for 7186EX(D) model number (7188XB(D) model number is 0)

       3            : for Receive and transmission interrupt

       125000UL     : for CAN baud

       0            : for BT0 of user defined baud

       0            : for BT1 of user defined baud

       0x00000000UL : for AccCode of CAN message filter

       0xffffffffUL : for AccMask of CAN message filter

    */

    switch (ret)
		{  

			/*Check if configuration is OK*/

	    case CAN_ResetError:

		    Print("Reset Error!\n\r");

		    return;

	    case CAN_SetACRError:

		    Print("\n\rSet ACR Error!");

		    return;

	    case CAN_SetAMRError:

		    Print("\n\rSet AMR Error!");

		    return;

	    case CAN_SetBaudRateError:

		    Print("\n\rSet Baud Rate Error!");

		    return;

	    case CAN_BaudNotSupport:

		    Print("\n\rBaud Rate Not Support!");

		    return;	

	    case CAN_ConfigError:

		    Print("\n\rConfiguration Failure!");

		    return;

	    default:

		    if (ret >0){

			    Print("Default=%d\n\r",ret);

			    return;

		    }

     } //end switch

    /********************************************************/

    /*                     Enable Irq                       */

    /********************************************************/

    CAN_CreateBuffer(0,1000);   //Create RX Buffer

    CAN_CreateBuffer(1,1000);   //Create TX Buffer

    CAN_InstallIrq(); //Set the interrupt function enable

    L1On(); /*Turn on LED1 for indicating that the demo is starting*/ 

    

     //Change to operational model

     //Send NMT to enter operational mode
			 
			SendCANNMT(0,0x01);
		 
			
	    /* 

	       When status of DI-channel is changed,the TxPDO is 

				 transmitted automatically when the transmission type is 255, based on the 

				 CANopen spec 401 .	    	 

	    */

	    

	    while(_quit)
	    {				
	    	      PrintFlag = 0; 
      	      Print("mode :(1)Write DO value (2)Set event time (3)Reset event time (0)exit ");
	    				Scanf("%d\n\r",&_choice);	
							PrintFlag = 1;
	    				switch(_choice)
	    				{	

	    					case 1:	    					

			    				//send PDO to write DO value
									PrintFlag = 0;			
									Print("Intput DO value (0xff) : ");
									Scanf("%x",&_hexValue);																			
									PrintFlag = 1;
									//send PDO (COB-ID (200h+node ID) used by RxPDO)

									SendCANPDO(_hexValue);									

	    					break;

	    					

	    					case 2:    						
									
									PrintFlag = 0;
			    				Print("Input Event Time values (ex:1000) :");
			    				Scanf("%d",&_hexValue2);
									PrintFlag = 1;
			    				//send SDO			    				

									SendCANSDO(0x2B,0x00,0x18,0x05,(unsigned char)_hexValue2,(unsigned char)(_hexValue2>>8),0x00,0x00);																																		

	    					break;

	    					

	    					case 3:	    					

			    				//send SDO

									SendCANSDO(0x2B,0x00,0x18,0x05,0x00,0x00,0x00,0x00);												

	    					break;

	    					

	    					case 0:	    						

									//send NMT to reset node

									SendCANNMT(0,0x81); 
									_quit=0;

				    		break;

	    					

	    					default:	

	    						Print("Wrong choice,Please try angain\n\r");	   						

	    					break;  

							}

						

	    }

  

	 L1Off();

	 L2Off();

	 L3Off();

	 

	 CAN_Restore();	//free resource

}



void ReceiveCANData(void)

{

			unsigned char MsgData[8];

			unsigned char MsgMode,MsgRTR,MsgDataLen;

			unsigned long MsgID,MsgUpperTimeStamp,MsgLowerTimeStamp;

			int ret,i;			

			/***************************************************************/

      /*  Receive CAN messages  from receive buffer or from CAN bus  */

      /***************************************************************/

      /*

      	*MsgMode :This parameter is used for get the ID type (11-bit or 29-bit ID) of   

        					a CAN message

      	*MsgID: This is for obtaining the ID of a CAN message.                

				*RTR: This is for obtaining the RTR of a CAN message.

							RTR value  Meaning 

							   0			 This CAN message is not a remote transmit request message.

							 	 1			 This CAN message is a remote transmit request message.  

				*DataLen: This is for obtaining the data length of a CAN message.

				*Data: This is for obtaining the Data of a CAN message. The Data buffer 

							 size must be 8 bytes. 								 

				*UpperTime: Get the time stamp of a CAN message. The time stamp 

										unit is us (micro second), This parameter only show the 

										upper part of time stamp.   

					      		Real time stamp = upper part * 0x1000000UL+lower part 

				*LowerTime: Get the lower part of time stamp of a CAN message. 

      */

			ret=GetCANMsg(&MsgMode,&MsgID,&MsgRTR,&MsgDataLen,MsgData,&MsgUpperTimeStamp,&MsgLowerTimeStamp);

		

	    if (!ret)

	    {

	      MsgCnt++;

	      //Print received information

		    Print("%lu(%lus-%luus):Mode=%d,ID=%lx,RTR=%d,Len=%d",MsgCnt,MsgUpperTimeStamp

			   ,MsgLowerTimeStamp,MsgMode,MsgID,MsgRTR,MsgDataLen);

		    if (MsgDataLen && !MsgRTR)

		    {

			    Print(",Data=");

			    for (i=0;i<MsgDataLen;i++)

			    {

				    Print("%x,",MsgData[i]);

			    }

			    Print("\n\r");

			  	//Print only received data 

			  	//For TxSDO 

			  	if((MsgID >=0x580) && (MsgID<=0x5FF))

			  		{

			  			switch(MsgData[0])

			  			{

			  				case 0x4F:

			  					Print("Data Value = %02Xh\n\r",MsgData[4]);

			  				break;

			  				

			  				case 0x4B:			  					

			  					Print("Data Value = %02X%02Xh\n\r",MsgData[5],MsgData[4]);

			  				break;

			  				

			  				case 0x47:

			  					Print("Data Value = %02X%02X%02Xh\n\r",MsgData[6],MsgData[5],MsgData[4]);

			  				break;		

			  					  					

			  				case 0x43:

			  					Print("Data Value = %02X%02X%02X%02Xh\n\r",MsgData[7],MsgData[6],MsgData[5],MsgData[4]);

			  				break;

			  			}

			  		}	

			  	  else if ((MsgID>=0x181 && MsgID<=0x18F)||(MsgID>=0x281 && MsgID<=0x28F)||(MsgID>=0x381 && MsgID<=0x38F)||(MsgID>=0x481 && MsgID<=0x48F))

			  		{//For TxPDO

			  			Print("Data value = ");

			  			for (i=0;i<MsgDataLen;i++)

			  			{

			  				if((MsgDataLen - (i+1))>0)

			  				{Print("%02Xh,",MsgData[i]);	}

			  				else

			  				{Print("%02Xh",MsgData[i]);	}

			  			}

			  			Print("\n\r");

			  		}

			  		else

			  			{

			  				;

			  			}

		    }

	    }

	    else

	    	{

		    switch(ret)

		    {

			    case CAN_DataLengthError:

				    Print("Reception Data Length Error!\n");

				    break;

			    case CAN_DataOverrun:

				    Print("Software Reception Data Buffer Overrun!\n");

				    break;

			    case CAN_ReceiveError:

				    Print("Receive data Error!\n");

				    break;

			    case CAN_ReceiveBufferEmpty: /*No message in receive buffer*/

				    break;

		    }

      }

}



void SendCANSDO(unsigned char cmd,unsigned char index_l,unsigned char index_h,unsigned char sub_index,

						    unsigned char D1,unsigned char D2,unsigned char D3,unsigned char D4)

{	

	int ret;

	MsgData[0]=cmd; 		//command specifier

	MsgData[1]=index_l; 		//index

	MsgData[2]=index_h; 		//index

	MsgData[3]=sub_index; 		//sub index 

	MsgData[4]=D1;//value 

	MsgData[5]=D2;

	MsgData[6]=D3;

	MsgData[7]=D4;

	ret=SendCANMsg(0,0x601,0,8,MsgData);	

	if (ret)

	 {

					    switch(ret)

					    {

						    case CAN_DataLengthError:

							    Print("Transmission Data Length Error!\n");

							    break;

						    case CAN_TransmitIncomplete:

							    Print("Transmission is incomplete!\n");

							    break;

						    case CAN_TransmitBufferLocked:

							    Print("CAN controller transmit Buffer is locked!\n");

							    break;

					    }

				

	 }

	ReceiveCANData();

}

void SendCANPDO(unsigned char D1)

{

	int ret;

	//send PDO (COB-ID (200h+node ID) used by RxPDO)

	MsgData[0]=D1; //Do value

	MsgData[1]=0x00; 

	MsgData[2]=0x00; 

	MsgData[3]=0x00; 

	MsgData[4]=0x00; 

	MsgData[5]=0x00;

	MsgData[6]=0x00;

	MsgData[7]=0x00;

	ret=SendCANMsg(0,0x201,0,8,MsgData);	

	

	 if (ret)

	    {

		    switch(ret)

		    {

			    case CAN_DataLengthError:

				    Print("Transmission Data Length Error!\n");

				    break;

			    case CAN_TransmitIncomplete:

				    Print("Transmission is incomplete!\n");

				    break;

			    case CAN_TransmitBufferLocked:

				    Print("CAN controller transmit Buffer is locked!\n");

				    break;

		    }

		    //break;

	    }

	ReceiveCANData();

}					    

void SendCANNMT(unsigned char RTR,unsigned char Cmd)

{

	int ret;

	if(RTR==0)

	{

			MsgData[0]=Cmd; //command specifier

			MsgData[1]=0x01; //node id

			MsgData[2]=0x00;

			MsgData[3]=0x00;

			MsgData[4]=0x00;

			MsgData[5]=0x00;

			MsgData[6]=0x00;

			MsgData[7]=0x00;

			ret=SendCANMsg(0,0x00,0,8,MsgData);						    

	}

	else

	{			

			MsgData[0]=0x00; //command specifier

			MsgData[1]=0x00; //node id

			MsgData[2]=0x00;

			MsgData[3]=0x00;

			MsgData[4]=0x00;

			MsgData[5]=0x00;

			MsgData[6]=0x00;

			MsgData[7]=0x00;

			ret=SendCANMsg(0,0x701,1,8,MsgData);						    		

	}

	 if (ret)

	    {

		    switch(ret)

		    {

			    case CAN_DataLengthError:

				    Print("Transmission Data Length Error!\n");

				    break;

			    case CAN_TransmitIncomplete:

				    Print("Transmission is incomplete!\n");

				    break;

			    case CAN_TransmitBufferLocked:

				    Print("CAN controller transmit Buffer is locked!\n");

				    break;

		    }

		    //break;

	    }

	ReceiveCANData();

}



